# $Id: tui.rb,v 1.1 2003/07/10 16:46:09 sdalu Exp $

require 'ncurses'

module TUI
    module Util
	def self.contstr(title, size, adjust=0)
	    str = if size <= 0
		  then ""
		  else if title.length > size
		       then if size < 2
			    then ".."[0,size]
			    else title[0,size-2] + ".."
			    end
		       else title
		       end
		  end
	    if    adjust == 0 then str
	    elsif adjust <  0 then str.ljust(size)
	    else                   str.rjust(size)
	    end
	end
    end

    def self.start 
	Ncurses.initscr
	Ncurses.cbreak			# provide unbuffered input
	Ncurses.noecho			# turn off input echoing
	Ncurses.nonl			# turn off newline translation
	Ncurses.curs_set(0)

	Ncurses.stdscr.intrflush(false)	# turn off flush-on-interrupt
	Ncurses.stdscr.keypad(true)	# turn on keypad mode
    end
    
    def self.cleanup
	Ncurses.echo
	Ncurses.nocbreak
	Ncurses.nl
	Ncurses.endwin
    end

    class Border
	Default = [0, 0, 0, 0, 0, 0, 0, 0]
    end

    class Container
	def initialize(height, width, y, x, border=nil, title=nil)
	    @border, @title         = border, title
	    @scr		    = nil
	    @height, @width, @y, @x = height, width, y, x
	end

	def innerHeight ; @border ? @height-2 : @height ; end
	def outerHeight ; @height                       ; end
	def innerWidth  ; @border ? @width-2  : @width  ; end
	def outerWidth  ; @width                        ; end
	def innerY      ; @border ? 1 : 0               ; end
	def outerY      ; @y                            ; end
	def innerX      ; @border ? 1 : 0               ; end
	def outerX      ; @x                            ; end
	
	def realize
	    return if @realized
	    @scr = Ncurses::WINDOW::new(@height, @width, @y, @x)
	    @scr.intrflush(false)	# turn off flush-on-interrupt
	    @scr.keypad(true)		# turn on keypad mode
	    @realized = true
	end

	def update
	    return unless @realized
	    return unless @border
	    @scr.border(* @border)

	    return unless @title
	    if @width > 6
		title = " " + Util.contstr(@title, @width-6) + " "
		off = (@width - title.length) / 2
		@scr.mvaddstr(0,  off,                title)
		@scr.mvaddch(0,  off - 1, Ncurses::ACS_RTEE)
		@scr.mvaddch(0,  off + title.length, Ncurses::ACS_LTEE)
	    end
	end

	def refresh
	    return unless @realized
	    @scr.refresh
	end
    end

    class StatusBox < Container
	def initialize(height, width, y, x, border=nil, title=nil)
	    super
	    @string = ""
	end

	def status=(str)
	    @string = str
	    update ; refresh
	end

	def update
	    super
	    return unless @realized
	    line = Util.contstr(@string, innerWidth-1, -1)
	    @scr.mvaddstr(innerY, innerX, line)
	end
    end


    class ListBox < Container
	def initialize(height, width, y, x, border=nil, title=nil)
	    super
	    @items = []
	end

	def []=(idx, value)
	    @items[idx] = value
	    update ; refresh
	end

	def update
	    super
	    return unless @realized
	    @scr.mvvline(innerY, innerWidth, Ncurses::ACS_CKBOARD, innerHeight)
	    @scr.mvaddch(innerY, innerWidth, ?# )
	    innerHeight.times { |i|
		if @items[i]
		    line = Util.contstr(@items[i], innerWidth-1, -1)
		    @scr.mvaddstr(innerY+i, innerX, line)
		end
	    }
 	end
    end
end
