#!/usr/local/bin/ruby

require 'thread'
require 'mutex_m'
require 'getoptlong'

require 'manager'
require 'param'

#ZC_BIN="/usr/local/zonecheck/bin/zc"
ZC_BIN="/usr/local/bin/zc"
THREAD_COUNT=5

#
# Identification
#
ZCMON_CVS_NAME	= %q$Name: ZC-2_0_0b6 $
ZCMON_NAME	= "ZCMonitor"
ZCMON_VERSION	= (Proc::new { 
                       n = ZCMON_CVS_NAME.split[1]
                       n = /^ZC-(.*)/.match(n) unless n.nil?
                       n = n[1]                unless n.nil?
                       n = n.gsub(/_/, ".")    unless n.nil?
                       
                       n || "<unreleased>"
                   }).call
PROGNAME	= File.basename($0)


#
# Constants
#
EXIT_OK		=  0    # Everything went fine
EXIT_FAILED	=  1    # The program completed but the result is negative
EXIT_ABORTED	=  2    # The user aborted the program before completion
EXIT_ERROR	=  3    # An error unrelated to the result occured
EXIT_USAGE	=  9    # The user didn't bother reading the man page


param = Param::new
begin
    opts = GetoptLong::new(
		[ '--help',     '-h',	GetoptLong::NO_ARGUMENT		],
		[ '--version',	'-V',	GetoptLong::NO_ARGUMENT		],
		[ '--input',	'-I',	GetoptLong::REQUIRED_ARGUMENT	],
		[ '--backend',	'-B',	GetoptLong::REQUIRED_ARGUMENT	],
		[ '--controler','-C',	GetoptLong::REQUIRED_ARGUMENT	])
    opts.quiet = true

    opts.each { |opt, arg|
	case opt
	when '--help' 
	    exit(EXIT_USAGE)
	when '--version' then puts 'yo' ; exit(EXIT_OK)
	when '--input'		then param.input	<< arg
	when '--backend'	then param.backend	<< arg
	when '--controler'	then param.controler	<< arg
	end
    }
rescue GetoptLong::InvalidOption, GetoptLong::MissingArgument
end



# Instanciate control method
def getCtrl(name)
    begin
	require "ctrl/#{name}"
    rescue LoadError => e
	puts "ERROR: control interface '#{name}' not supported"
	exit EXIT_ERROR
    end
    eval "Ctrl::#{name.upcase}"
end

# Instanciate control method
def getInput(name)
    begin
	require "input/#{name}"
    rescue LoadError => e
	puts "ERROR: input interface '#{name}' not supported"
	exit EXIT_ERROR
    end
    eval "Input::#{name.upcase}"
end


# Instanciate control method
def getBackend(name)
    begin
	require "backend/#{name}"
    rescue LoadError => e
	puts "ERROR: bckend interface '#{name}' not supported"
	exit EXIT_ERROR
    end
    eval "Backend::#{name.upcase}"
end



manager = Manager::new


queue = Queue::new


manager.ctrl	= getCtrl(param.controler['klass'] || 'std')::new(param.controler)
manager.input	= getInput(param.input['klass'] || 'custom')::new(param.input)
manager.backend	= getBackend(param.backend['klass'] || 'fs')::new(param.backend)



manager.start
manager.join
exit 0










ctrlClass = getCtrlClass("std")
begin
    ctrlClass.start
    ctrl = ctrlClass::new(THREAD_COUNT)



    File::open("zc.log", "w") { |io|
	io.extend Mutex_m

	thrlist = []
	THREAD_COUNT.times { |thridx|
	    thrlist << Thread::new {

		while true
		    # Jitter
		    sleep(rand(2))
		    
		    # Retrieve information about the zone to test
		    coef, config, zone = queue.pop(false)
		    
		    # Display current status
		    ctrl.setWorker(thridx, "%2d: %s" % [ thridx, zone ])
		    
		    # Create ZoneCheck handler
		    nzc =  ZoneCheck::new(zone, config)

		    # Information about the file location
		    dirlist  = []
		    subzone  = zone.split(/\./)
		    subzone.each_index { |i| 
			dirlist << subzone[i..-1].join('.') }
		    dirlist.reverse!

		    tstamp   = Time::now.strftime("%Y%m%d")
		    filename = "#{dirlist.join('/')}/#{tstamp}.html"

		    # Create directory hierarchie
		    dirpath = []
		    dirlist.each { |dir| 
			dirpath << dir
			begin
			    Dir::mkdir(dirpath.join('/'))
			rescue Errno::EEXIST
			end
		    }

		    # Launch ZoneCheck and store result
		    File::open(filename, "w") { |fio|
			nzc.launch(nil, fio, fio)
		    }
		    
		    # Log information summary
		    io.synchronize {
			io.puts "ZONE=#{zone}"
			io.puts "COEF=#{coef}"
			io.puts "CMD =#{nzc.command.join(' ')}"
			if nzc.exitcode
			then io.puts "NEW =#{nzc.exitcode}"
			else io.puts "NEW =#{nzc.signal} (SIG)"
			end
			io.puts ""
			io.flush
		    }
		end
	    }
	}

	thrlist.each { |thr| thr.join }
    }
    
    

    exit 0

ensure
    ctrlClass.cleanup
end
